#!/usr/bin/env python
# $Id$
''' Output lesser version of an rpm when multiple versions are installed '''
import rpm
import re

BLACKLIST = re.compile('kernel|kernel-devel|^gpg-pubkey.*')

def rpm_query():
    ''' Query rpm database and return dictionary of all installed packages '''
    pkg_dict = {}
    trans = rpm.TransactionSet()
    matched_index = trans.dbMatch()
    for header in matched_index:
        if pkg_dict.has_key(header['name']):
            pkg_dict[header['name']].append("%s-%s.%s" % ( 
                                                      header['version'],
                                                      header['release'], 
                                                      header['arch']
                                                    ) )
        else:
            pkg_dict[header['name']] = [ ("%s-%s.%s" % ( 
                                                    header['version'],
                                                    header['release'], 
                                                    header['arch']
                                                   ) )]
    return pkg_dict

def get_lesser_version(packagename):
    ''' Return lesser version of given package name '''
    trans = rpm.TransactionSet()
    matched_index = trans.dbMatch('name', packagename)
    if matched_index.count() == 1:
        return None
    old_package = matched_index.next()
    for header in matched_index:
        if old_package['arch'] == header['arch']:
            if rpm.versionCompare(old_package, header) == 1:
                old_package = header
            elif rpm.versionCompare(old_package, header) == 0:
                old_package = None
        else:
            old_package = None
    return old_package

def get_all_lesser_versions():
    ''' Return lesser version for all packages installed '''
    for each in rpm_query().keys():
        if not BLACKLIST.match(each):
            dupes = get_lesser_version(each)
            if dupes: 
                yield "%s-%s-%s" % (
                                    dupes['name'], 
                                    dupes['version'], 
                                    dupes['release']
                                   )

if __name__ == '__main__':
    for pkg in get_all_lesser_versions():
        print pkg

# vim:syntax=python
# vim:sw=4:softtabstop=4:expandtab
